@@ -1,14 +1,15 @@
-# $Id: Simple.pm,v 1.40 2007/08/15 10:36:48 grantm Exp $
-
 package XML::Simple;
-
+$XML::Simple::VERSION = '2.21';
 =head1 NAME
 
-XML::Simple - Easy API to maintain XML (esp config files)
+XML::Simple - An API for simple XML files
 
 =head1 SYNOPSIS
 
-    use XML::Simple;
+You really don't want to use this module in new code.  If you ignore this
+warning and use it anyway, the C<qw(:strict)> mode will save you a little pain.
+
+    use XML::Simple qw(:strict);
 
     my $ref = XMLin([<xml file or string>] [, <options>]);
 
@@ -16,9 +17,9 @@ XML::Simple - Easy API to maintain XML (esp config files)
 
 Or the object oriented way:
 
-    require XML::Simple;
+    require XML::Simple qw(:strict);
 
-    my $xs = XML::Simple->new(options);
+    my $xs = XML::Simple->new([<options>]);
 
     my $ref = $xs->XMLin([<xml file or string>] [, <options>]);
 
@@ -26,11 +27,8 @@ Or the object oriented way:
 
 (or see L<"SAX SUPPORT"> for 'the SAX way').
 
-To catch common errors:
-
-    use XML::Simple qw(:strict);
-
-(see L<"STRICT MODE"> for more details).
+Note, in these examples, the square brackets are used to denote optional items
+not to imply items should be supplied in arrayrefs.
 
 =cut
 
@@ -40,6 +38,8 @@ To catch common errors:
 # Load essentials here, other modules loaded on demand later
 
 use strict;
+use warnings;
+use warnings::register;
 use Carp;
 require Exporter;
 
@@ -53,20 +53,19 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $PREFERRED_PARSER);
 @ISA               = qw(Exporter);
 @EXPORT            = qw(XMLin XMLout);
 @EXPORT_OK         = qw(xml_in xml_out);
-$VERSION           = '2.18';
 $PREFERRED_PARSER  = undef;
 
-my $StrictMode     = 0;
+my %StrictMode     = ();
 
 my @KnownOptIn     = qw(keyattr keeproot forcecontent contentkey noattr
                         searchpath forcearray cache suppressempty parseropts
                         grouptags nsexpand datahandler varattr variables
-                        normalisespace normalizespace valueattr);
+                        normalisespace normalizespace valueattr strictmode);
 
 my @KnownOptOut    = qw(keyattr keeproot contentkey noattr
                         rootname xmldecl outputfile noescape suppressempty
                         grouptags nsexpand handler noindent attrindent nosort
-                        valueattr numericescape);
+                        valueattr numericescape strictmode);
 
 my @DefKeyAttr     = qw(name key id);
 my $DefRootName    = qq(opt);
@@ -91,8 +90,9 @@ my %MemCopyCache   = ();
 
 sub import {
   # Handle the :strict tag
-  
-  $StrictMode = 1 if grep(/^:strict$/, @_);
+
+  my($calling_package) = caller();
+  _strict_mode_for_caller(1) if grep(/^:strict$/, @_);
 
   # Pass everything else to Exporter.pm
 
@@ -113,9 +113,11 @@ sub new {
   }
 
   my %known_opt;
-  @known_opt{@KnownOptIn, @KnownOptOut} = (undef) x 100;
+  @known_opt{@KnownOptIn, @KnownOptOut} = ();
 
   my %raw_opt = @_;
+  $raw_opt{strictmode} = _strict_mode_for_caller()
+    unless exists $raw_opt{strictmode};
   my %def_opt;
   while(my($key, $val) = each %raw_opt) {
     my $lkey = lc($key);
@@ -129,6 +131,27 @@ sub new {
 }
 
 
+##############################################################################
+# Sub: _strict_mode_for_caller()
+#
+# Gets or sets the XML::Simple :strict mode flag for the calling namespace.
+# Walks back through call stack to find the calling namespace and sets the
+# :strict mode flag for that namespace if an argument was supplied and returns
+# the flag value if not.
+#
+
+sub _strict_mode_for_caller {
+  my $set_mode = @_;
+  my $frame = 1;
+  while(my($package) = caller($frame++)) {
+    next if $package eq 'XML::Simple';
+    $StrictMode{$package} = 1 if $set_mode;
+    return $StrictMode{$package};
+  }
+  return(0);
+}
+
+
 ##############################################################################
 # Sub: _get_object()
 #
@@ -144,7 +167,7 @@ sub _get_object {
   else {
     $self = XML::Simple->new();
   }
-  
+
   return $self;
 }
 
@@ -274,7 +297,7 @@ sub parse_string {
 ##############################################################################
 # Method: default_config_file()
 #
-# Returns the name of the XML file to parse if no filename (or XML string) 
+# Returns the name of the XML file to parse if no filename (or XML string)
 # was provided.
 #
 
@@ -286,7 +309,7 @@ sub default_config_file {
   my($basename, $script_dir, $ext) = File::Basename::fileparse($0, '\.[^\.]+');
 
   # Add script directory to searchpath
-  
+
   if($script_dir) {
     unshift(@{$self->{opt}->{searchpath}}, $script_dir);
   }
@@ -298,14 +321,17 @@ sub default_config_file {
 ##############################################################################
 # Method: build_simple_tree()
 #
-# Builds a 'tree' data structure as provided by XML::Parser and then 
+# Builds a 'tree' data structure as provided by XML::Parser and then
 # 'simplifies' it as specified by the various options in effect.
 #
 
 sub build_simple_tree {
   my $self = shift;
 
-  my $tree = $self->build_tree(@_);
+  my $tree = eval {
+    $self->build_tree(@_);
+  };
+  Carp::croak("$@XML::Simple called") if $@;
 
   return $self->{opt}->{keeproot}
          ? $self->collapse({}, @$tree)
@@ -356,7 +382,7 @@ sub build_tree {
   $XML::SAX::ParserPackage = $preferred_parser if($preferred_parser);
 
   my $sp = XML::SAX::ParserFactory->parser(Handler => $self);
-  
+
   $self->{nocollapse} = 1;
   my($tree);
   if($filename) {
@@ -405,10 +431,8 @@ sub build_tree_xml_parser {
   my($tree);
   if($filename) {
     # $tree = $xp->parsefile($filename);  # Changed due to prob w/mod_perl
-    local(*XML_FILE);
-    open(XML_FILE, '<', $filename) || croak qq($filename - $!);
-    $tree = $xp->parse(*XML_FILE);
-    close(XML_FILE);
+    open(my $xfh, '<', $filename) || croak qq($filename - $!);
+    $tree = $xp->parse($xfh);
   }
   else {
     $tree = $xp->parse($$string);
@@ -439,7 +463,7 @@ sub cache_write_storable {
     # If the following line fails for you, your Storable.pm is old - upgrade
     Storable::lock_nstore($data, $cachefile);
   }
-  
+
 }
 
 
@@ -453,21 +477,21 @@ sub cache_write_storable {
 
 sub cache_read_storable {
   my($self, $filename) = @_;
-  
+
   my $cachefile = $self->storable_filename($filename);
 
   return unless(-r $cachefile);
   return unless((stat($cachefile))[9] > (stat($filename))[9]);
 
   require Storable;           # We didn't need it until now
-  
+
   if ('VMS' eq $^O) {
     return(Storable::retrieve($cachefile));
   }
   else {
     return(Storable::lock_retrieve($cachefile));
   }
-  
+
 }
 
 
@@ -509,12 +533,12 @@ sub cache_write_memshare {
 
 sub cache_read_memshare {
   my($self, $filename) = @_;
-  
+
   return unless($MemShareCache{$filename});
   return unless($MemShareCache{$filename}->[0] > (stat($filename))[9]);
 
   return($MemShareCache{$filename}->[1]);
-  
+
 }
 
 
@@ -529,7 +553,7 @@ sub cache_write_memcopy {
   my($self, $data, $filename) = @_;
 
   require Storable;           # We didn't need it until now
-  
+
   $MemCopyCache{$filename} = [time(), Storable::dclone($data)];
 }
 
@@ -543,12 +567,12 @@ sub cache_write_memcopy {
 
 sub cache_read_memcopy {
   my($self, $filename) = @_;
-  
+
   return unless($MemCopyCache{$filename});
   return unless($MemCopyCache{$filename}->[0] > (stat($filename))[9]);
 
   return(Storable::dclone($MemCopyCache{$filename}->[1]));
-  
+
 }
 
 
@@ -595,7 +619,7 @@ sub XMLout {
       $self->{opt}->{rootname} = $keys[0];
     }
   }
-  
+
   # Ensure there are no top level attributes if we're not adding root elements
 
   elsif($self->{opt}->{rootname} eq '') {
@@ -634,12 +658,11 @@ sub XMLout {
       return($fh->print($xml));
     }
     else {
-      local(*OUT);
-      open(OUT, '>', "$self->{opt}->{outputfile}") ||
+      open(my $out, '>', "$self->{opt}->{outputfile}") ||
         croak "open($self->{opt}->{outputfile}): $!";
-      binmode(OUT, ':utf8') if($] >= 5.008);
-      print OUT $xml || croak "print: $!";
-      close(OUT);
+      binmode($out, ':utf8') if($] >= 5.008);
+      print $out $xml or croak "print: $!";
+      close $out or croak "close: $!";
     }
   }
   elsif($self->{opt}->{handler}) {
@@ -678,7 +701,7 @@ sub handle_options  {
 
   # Determine valid options based on context
 
-  my %known_opt; 
+  my %known_opt;
   if($dirn eq 'in') {
     @known_opt{@KnownOptIn} = @KnownOptIn;
   }
@@ -716,7 +739,7 @@ sub handle_options  {
 
 
   # Set sensible defaults if not supplied
-  
+
   if(exists($opt->{rootname})) {
     unless(defined($opt->{rootname})) {
       $opt->{rootname} = '';
@@ -725,7 +748,7 @@ sub handle_options  {
   else {
     $opt->{rootname} = $DefRootName;
   }
-  
+
   if($opt->{xmldecl}  and  $opt->{xmldecl} eq '1') {
     $opt->{xmldecl} = $DefXmlDecl;
   }
@@ -767,9 +790,9 @@ sub handle_options  {
         unless($self->can($method));
     }
   }
-  
+
   if(exists($opt->{parseropts})) {
-    if($^W) {
+    if(warnings::enabled()) {
       carp "Warning: " .
            "'ParserOpts' is deprecated, contact the author if you need it";
     }
@@ -778,7 +801,7 @@ sub handle_options  {
     $opt->{parseropts} = [ ];
   }
 
-  
+
   # Special cleanup for {forcearray} which could be regex, arrayref or boolean
   # or left to default to 0
 
@@ -809,7 +832,7 @@ sub handle_options  {
     }
   }
   else {
-    if($StrictMode  and  $dirn eq 'in') {
+    if($opt->{strictmode}  and  $dirn eq 'in') {
       croak "No value specified for 'ForceArray' option in call to XML$dirn()";
     }
     $opt->{forcearray} = 0;
@@ -827,14 +850,14 @@ sub handle_options  {
 
         $opt->{keyattr} = { %{$opt->{keyattr}} };
 
-        
+
         # Convert keyattr => { elem => '+attr' }
-        # to keyattr => { elem => [ 'attr', '+' ] } 
+        # to keyattr => { elem => [ 'attr', '+' ] }
 
         foreach my $el (keys(%{$opt->{keyattr}})) {
           if($opt->{keyattr}->{$el} =~ /^(\+|-)?(.*)$/) {
             $opt->{keyattr}->{$el} = [ $2, ($1 ? $1 : '') ];
-            if($StrictMode  and  $dirn eq 'in') {
+            if($opt->{strictmode}  and  $dirn eq 'in') {
               next if($opt->{forcearray} == 1);
               next if(ref($opt->{forcearray}) eq 'HASH'
                       and $opt->{forcearray}->{$el});
@@ -857,7 +880,7 @@ sub handle_options  {
     }
   }
   else  {
-    if($StrictMode) {
+    if($opt->{strictmode}) {
       croak "No value specified for 'KeyAttr' option in call to XML$dirn()";
     }
     $opt->{keyattr} = [ @DefKeyAttr ];
@@ -892,10 +915,10 @@ sub handle_options  {
     croak "Illegal value for 'Variables' option - expected a hashref";
   }
 
-  if($opt->{variables}) { 
+  if($opt->{variables}) {
     $self->{_var_values} = { %{$opt->{variables}} };
   }
-  elsif($opt->{varattr}) { 
+  elsif($opt->{varattr}) {
     $self->{_var_values} = {};
   }
 
@@ -976,10 +999,10 @@ sub collapse {
 
 
   # Start with the hash of attributes
-  
+
   my $attr  = shift;
   if($self->{opt}->{noattr}) {                    # Discard if 'noattr' set
-    $attr = {};
+    $attr = $self->new_hashref;
   }
   elsif($self->{opt}->{normalisespace} == 2) {
     while(my($key, $value) = each %$attr) {
@@ -992,7 +1015,7 @@ sub collapse {
 
   if(my $var = $self->{_var_values}) {
     while(my($key, $val) = each(%$attr)) {
-      $val =~ s{\$\{([\w.]+)\}}{ $self->get_var($1) }ge;
+      $val =~ s^\$\{([\w.]+)\}^ $self->get_var($1) ^ge;
       $attr->{$key} = $val;
     }
   }
@@ -1014,6 +1037,7 @@ sub collapse {
   while(@_) {
     $key = shift;
     $val = shift;
+    $val = '' if not defined $val;
 
     if(ref($val)) {
       $val = $self->collapse(@$val);
@@ -1027,14 +1051,14 @@ sub collapse {
 
       # do variable substitutions
 
-      if(my $var = $self->{_var_values}) { 
-        $val =~ s{\$\{(\w+)\}}{ $self->get_var($1) }ge;
+      if(my $var = $self->{_var_values}) {
+        $val =~ s^\$\{(\w+)\}^ $self->get_var($1) ^ge;
       }
 
-      
+
       # look for variable definitions
 
-      if(my $var = $self->{opt}->{varattr}) { 
+      if(my $var = $self->{opt}->{varattr}) {
         if(exists $attr->{$var}) {
           $self->set_var($attr->{$var}, $val);
         }
@@ -1044,7 +1068,7 @@ sub collapse {
       # Collapse text content in element with no attributes to a string
 
       if(!%$attr  and  !@_) {
-        return($self->{opt}->{forcecontent} ? 
+        return($self->{opt}->{forcecontent} ?
           { $self->{opt}->{contentkey} => $val } : $val
         );
       }
@@ -1066,10 +1090,10 @@ sub collapse {
       $attr->{$key} = [ $val ];
     }
     else {
-      if( $key ne $self->{opt}->{contentkey} 
+      if( $key ne $self->{opt}->{contentkey}
           and (
             ($self->{opt}->{forcearray} == 1)
-            or ( 
+            or (
               (ref($self->{opt}->{forcearray}) eq 'HASH')
               and (
                 $self->{opt}->{forcearray}->{$key}
@@ -1118,8 +1142,8 @@ sub collapse {
   # Fold hashes containing a single anonymous array up into just the array
 
   my $count = scalar keys %$attr;
-  if($count == 1 
-     and  exists $attr->{anon}  
+  if($count == 1
+     and  exists $attr->{anon}
      and  UNIVERSAL::isa($attr->{anon}, 'ARRAY')
   ) {
     return($attr->{anon});
@@ -1240,7 +1264,7 @@ sub array_to_hash {
           if($self->{opt}->{normalisespace} == 1);
         $self->die_or_warn("<$name> element has non-unique value in '$key' key attribute: $val")
           if(exists($hashref->{$val}));
-        $hashref->{$val} = { %{$arrayref->[$i]} };
+        $hashref->{$val} = $self->new_hashref( %{$arrayref->[$i]} );
         $hashref->{$val}->{"-$key"} = $hashref->{$val}->{$key} if($flag eq '-');
         delete $hashref->{$val}->{$key} unless($flag eq '+');
       }
@@ -1273,7 +1297,7 @@ sub array_to_hash {
             if($self->{opt}->{normalisespace} == 1);
           $self->die_or_warn("<$name> element has non-unique value in '$key' key attribute: $val")
             if(exists($hashref->{$val}));
-          $hashref->{$val} = { %{$arrayref->[$i]} };
+          $hashref->{$val} = $self->new_hashref( %{$arrayref->[$i]} );
           delete $hashref->{$val}->{$key};
           next ELEMENT;
         }
@@ -1282,13 +1306,13 @@ sub array_to_hash {
       return($arrayref);    # No keyfield matched
     }
   }
-  
+
   # collapse any hashes which now only have a 'content' key
 
   if($self->{opt}->{collapseagain}) {
     $hashref = $self->collapse_content($hashref);
   }
- 
+
   return($hashref);
 }
 
@@ -1299,16 +1323,18 @@ sub array_to_hash {
 # Takes a diagnostic message and does one of three things:
 # 1. dies if strict mode is enabled
 # 2. warns if warnings are enabled but strict mode is not
-# 3. ignores message and resturns silently if neither strict mode nor warnings
+# 3. ignores message and returns silently if neither strict mode nor warnings
 #    are enabled
-# 
+#
 
 sub die_or_warn {
   my $self = shift;
   my $msg  = shift;
 
-  croak $msg if($StrictMode);
-  carp "Warning: $msg" if($^W);
+  croak $msg if($self->{opt}->{strictmode});
+  if(warnings::enabled()) {
+    carp "Warning: $msg";
+  }
 }
 
 
@@ -1317,7 +1343,7 @@ sub die_or_warn {
 #
 # This is a hook routine for overriding in a sub-class.  Some people believe
 # that using Tie::IxHash here will solve order-loss problems.
-# 
+#
 
 sub new_hashref {
   my $self = shift;
@@ -1330,10 +1356,10 @@ sub new_hashref {
 # Method: collapse_content()
 #
 # Helper routine for array_to_hash
-# 
+#
 # Arguments expected are:
 # - an XML::Simple object
-# - a hasref
+# - a hashref
 # the hashref is a former array, turned into a hash by array_to_hash because
 # of the presence of key attributes
 # at this point collapse_content avoids over-complicated structures like
@@ -1347,7 +1373,7 @@ sub new_hashref {
 
 sub collapse_content {
   my $self       = shift;
-  my $hashref    = shift; 
+  my $hashref    = shift;
 
   my $contentkey = $self->{opt}->{contentkey};
 
@@ -1366,14 +1392,14 @@ sub collapse_content {
 
   return $hashref;
 }
-  
+
 
 ##############################################################################
 # Method: value_to_xml()
 #
 # Helper routine for XMLout() - recurses through a data structure building up
 # and returning an XML representation of that structure as a string.
-# 
+#
 # Arguments expected are:
 # - the data structure to be encoded (usually a reference)
 # - the XML tag name to use for this item
@@ -1400,7 +1426,7 @@ sub value_to_xml {
 
 
   # Convert to XML
-  
+
   if(ref($ref)) {
     croak "circular data structures not supported"
       if(grep($_ == $ref, @{$self->{_ancestors}}));
@@ -1445,7 +1471,9 @@ sub value_to_xml {
       $ref = $self->copy_hash($ref);
       while(my($key, $val) = each %$ref) {
         if($self->{opt}->{grouptags}->{$key}) {
-          $ref->{$key} = { $self->{opt}->{grouptags}->{$key} => $val };
+          $ref->{$key} = $self->new_hashref(
+            $self->{opt}->{grouptags}->{$key} => $val
+          );
         }
       }
     }
@@ -1463,7 +1491,7 @@ sub value_to_xml {
 
       if(exists($ref->{xmlns})) {
         $self->{nsup}->declare_prefix('', $ref->{xmlns});
-        $nsdecls .= qq( xmlns="$ref->{xmlns}"); 
+        $nsdecls .= qq( xmlns="$ref->{xmlns}");
         delete($ref->{xmlns});
       }
       $default_ns_uri = $self->{nsup}->get_uri('');
@@ -1476,7 +1504,7 @@ sub value_to_xml {
         if($uri) {
           if($uri eq $xmlns_ns) {
             $self->{nsup}->declare_prefix($lname, $ref->{$qname});
-            $nsdecls .= qq( xmlns:$lname="$ref->{$qname}"); 
+            $nsdecls .= qq( xmlns:$lname="$ref->{$qname}");
             delete($ref->{$qname});
           }
         }
@@ -1498,7 +1526,7 @@ sub value_to_xml {
               # $prefix = $self->{nsup}->get_prefix($uri);
               $prefix = $self->{ns_prefix}++;
               $self->{nsup}->declare_prefix($prefix, $uri);
-              $nsdecls .= qq( xmlns:$prefix="$uri"); 
+              $nsdecls .= qq( xmlns:$prefix="$uri");
             }
             $ref->{"$prefix:$lname"} = $ref->{$qname};
             delete($ref->{$qname});
@@ -1524,7 +1552,7 @@ sub value_to_xml {
           unless(exists($self->{opt}->{suppressempty})
              and !defined($self->{opt}->{suppressempty})
           ) {
-            carp 'Use of uninitialized value' if($^W);
+            carp 'Use of uninitialized value' if warnings::enabled();
           }
           if($key eq $self->{opt}->{contentkey}) {
             $text_content = '';
@@ -1534,11 +1562,13 @@ sub value_to_xml {
           }
         }
 
-        if(!ref($value)  
+        if(!ref($value)
            and $self->{opt}->{valueattr}
            and $self->{opt}->{valueattr}->{$key}
         ) {
-          $value = { $self->{opt}->{valueattr}->{$key} => $value };
+          $value = $self->new_hashref(
+            $self->{opt}->{valueattr}->{$key} => $value
+          );
         }
 
         if(ref($value)  or  $self->{opt}->{noattr}) {
@@ -1806,7 +1836,7 @@ sub characters {
   my $text  = $chars->{Data};
   my $clist = $self->{curlist};
   my $pos = $#$clist;
-  
+
   if ($pos > 0 and $clist->[$pos - 1] eq '0') {
     $clist->[$pos] .= $text;
   }
@@ -1839,7 +1869,7 @@ sub end_document {
 
 
   # Or collapse it before returning it to SAX parser class
-  
+
   if($self->{opt}->{keeproot}) {
     $tree = $self->collapse({}, @$tree);
   }
@@ -1861,10 +1891,24 @@ sub end_document {
 
 __END__
 
+=head1 STATUS OF THIS MODULE
+
+The use of this module in new code is discouraged.  Other modules are available
+which provide more straightforward and consistent interfaces.  In particular,
+L<XML::LibXML> is highly recommended and L<XML::Twig> is an excellent
+alternative.
+
+The major problems with this module are the large number of options (some of
+which have unfortunate defaults) and the arbitrary ways in which these options
+interact - often producing unexpected results.
+
+Patches with bug fixes and documentation fixes are welcome, but new features
+are unlikely to be added.
+
 =head1 QUICK START
 
 Say you have a script called B<foo> and a file of configuration options
-called B<foo.xml> containing this:
+called B<foo.xml> containing the following:
 
   <config logdir="/var/log/foo/" debugfile="/tmp/foo.debug">
     <server name="sahara" osname="solaris" osversion="2.6">
@@ -1882,14 +1926,14 @@ called B<foo.xml> containing this:
 
 The following lines of code in B<foo>:
 
-  use XML::Simple;
+  use XML::Simple qw(:strict);
 
-  my $config = XMLin();
+  my $config = XMLin(undef, KeyAttr => { server => 'name' }, ForceArray => [ 'server', 'address' ]);
 
 will 'slurp' the configuration options into the hashref $config (because no
-arguments are passed to C<XMLin()> the name and location of the XML file will
-be inferred from name and location of the script).  You can dump out the
-contents of the hashref using Data::Dumper:
+filename or XML string was passed as the first argument to C<XMLin()> the name
+and location of the XML file will be inferred from name and location of the
+script).  You can dump out the contents of the hashref using Data::Dumper:
 
   use Data::Dumper;
 
@@ -1910,7 +1954,7 @@ brevity):
           'gobi'          => {
               'osversion'     => '6.5',
               'osname'        => 'irix',
-              'address'       => '10.0.0.102'
+              'address'       => [ '10.0.0.102' ]
           },
           'kalahari'      => {
               'osversion'     => '2.0.34',
@@ -1928,7 +1972,18 @@ similarly, the second address on the server 'kalahari' could be referenced as:
 
   print $config->{server}->{kalahari}->{address}->[1];
 
-What could be simpler?  (Rhetorical).
+Note: If the mapping between the output of Data::Dumper and the print
+statements above is not obvious to you, then please refer to the 'references'
+tutorial (AKA: "Mark's very short tutorial about references") at L<perlreftut>.
+
+In this example, the C<< ForceArray >> option was used to list elements that
+might occur multiple times and should therefore be represented as arrayrefs
+(even when only one element is present).
+
+The C<< KeyAttr >> option was used to indicate that each C<< <server> >>
+element has a unique identifier in the C<< name >> attribute.  This allows you
+to index directly to a particular server record using the name as a hash key
+(as shown above).
 
 For simple requirements, that's really all there is to it.  If you want to
 store your XML in a different directory or file, or pass it in as a string or
@@ -1947,7 +2002,7 @@ case, you might want to read L<"WHERE TO FROM HERE?">.
 
 The XML::Simple module provides a simple API layer on top of an underlying XML
 parsing module (either XML::Parser or one of the SAX2 parser modules).  Two
-functions are exported: C<XMLin()> and C<XMLout()>.  Note: you can explicity
+functions are exported: C<XMLin()> and C<XMLout()>.  Note: you can explicitly
 request the lower case versions of the function names: C<xml_in()> and
 C<xml_out()>.
 
@@ -2006,7 +2061,7 @@ An IO::Handle object will be read to EOF and its contents parsed. eg:
 
 Takes a data structure (generally a hashref) and returns an XML encoding of
 that structure.  If the resulting XML is parsed using C<XMLin()>, it should
-return a data structure equivalent to the original (see caveats below). 
+return a data structure equivalent to the original (see caveats below).
 
 The C<XMLout()> function can also be used to output the XML as SAX events
 see the C<Handler> option and L<"SAX SUPPORT"> for more details).
@@ -2021,7 +2076,7 @@ with a leading '-' which would not be valid XML).
 
 Some care is required in creating data structures which will be passed to
 C<XMLout()>.  Hash keys from the data structure will be encoded as either XML
-element names or attribute names.  Therefore, you should use hash key names 
+element names or attribute names.  Therefore, you should use hash key names
 which conform to the relatively strict XML naming rules:
 
 Names in XML must begin with a letter.  The remaining characters may be
@@ -2033,7 +2088,7 @@ namespaces when teamed with a SAX Parser).
 You can use other punctuation characters in hash values (just not in hash
 keys) however B<XML::Simple> does not support dumping binary data.
 
-If you break these rules, the current implementation of C<XMLout()> will 
+If you break these rules, the current implementation of C<XMLout()> will
 simply emit non-compliant XML which will be rejected if you try to read it
 back in.  (A later version of B<XML::Simple> might take a more proactive
 approach).
@@ -2041,9 +2096,9 @@ approach).
 Note also that although you can nest hashes and arrays to arbitrary levels,
 circular data structures are not supported and will cause C<XMLout()> to die.
 
-If you wish to 'round-trip' arbitrary data structures from Perl to XML and back 
+If you wish to 'round-trip' arbitrary data structures from Perl to XML and back
 to Perl, then you should probably disable array folding (using the KeyAttr
-option) both with C<XMLout()> and with C<XMLin()>.  If you still don't get the 
+option) both with C<XMLout()> and with C<XMLin()>.  If you still don't get the
 expected results, you may prefer to use L<XML::Dumper> which is designed for
 exactly that purpose.
 
@@ -2182,13 +2237,13 @@ instead of:
 C<XMLout()> will also honour the value of this option when converting a hashref
 to XML.
 
-You can also prefix your selected key name with a '-' character to have 
+You can also prefix your selected key name with a '-' character to have
 C<XMLin()> try a little harder to eliminate unnecessary 'content' keys after
 array folding.  For example:
 
   XMLin(
-    '<opt><item name="one">First</item><item name="two">Second</item></opt>', 
-    KeyAttr => {item => 'name'}, 
+    '<opt><item name="one">First</item><item name="two">Second</item></opt>',
+    KeyAttr => {item => 'name'},
     ForceArray => [ 'item' ],
     ContentKey => '-content'
   )
@@ -2247,7 +2302,7 @@ instead of this (the default):
 
 This option is especially useful if the data structure is likely to be written
 back out as XML and the default behaviour of rolling single nested elements up
-into attributes is not desirable. 
+into attributes is not desirable.
 
 If you are using the array folding feature, you should almost certainly enable
 this option.  If you do not, single nested elements will not be parsed to
@@ -2336,15 +2391,15 @@ for elements as well as attributes.
 
 =head2 Handler => object_ref I<# out - SAX only>
 
-Use the 'Handler' option to have C<XMLout()> generate SAX events rather than 
+Use the 'Handler' option to have C<XMLout()> generate SAX events rather than
 returning a string of XML.  For more details see L<"SAX SUPPORT"> below.
 
 Note: the current implementation of this option generates a string of XML
 and uses a SAX parser to translate it into SAX events.  The normal encoding
-rules apply here - your data must be UTF8 encoded unless you specify an 
+rules apply here - your data must be UTF8 encoded unless you specify an
 alternative encoding via the 'XMLDecl' option; and by the time the data reaches
 the handler object, it will be in UTF8 form regardless of the encoding you
-supply.  A future implementation of this option may generate the events 
+supply.  A future implementation of this option may generate the events
 directly.
 
 =head2 KeepRoot => 1 I<# in+out - handy>
@@ -2412,7 +2467,7 @@ supplied.  C<XMLout()> will use the first attribute name supplied when
 'unfolding' a hash into an array.
 
 Note 1: The default value for 'KeyAttr' is ['name', 'key', 'id'].  If you do
-not want folding on input or unfolding on output you must setting this option
+not want folding on input or unfolding on output you must set this option
 to an empty list to disable the feature.
 
 Note 2: If you wish to use this option, you should also enable the
@@ -2422,16 +2477,17 @@ rolled up into a scalar rather than an array and therefore will not be folded
 
 =head2 KeyAttr => { list } I<# in+out - important>
 
-This alternative (and preferred) method of specifiying the key attributes
+This alternative (and preferred) method of specifying the key attributes
 allows more fine grained control over which elements are folded and on which
 attributes.  For example the option 'KeyAttr => { package => 'id' } will cause
 any package elements to be folded on the 'id' attribute.  No other elements
-which have an 'id' attribute will be folded at all. 
+which have an 'id' attribute will be folded at all.
 
 Note: C<XMLin()> will generate a warning (or a fatal error in L<"STRICT MODE">)
 if this syntax is used and an element which does not have the specified key
 attribute is encountered (eg: a 'package' element without an 'id' attribute, to
-use the example above).  Warnings will only be generated if B<-w> is in force.
+use the example above).  Warnings can be suppressed with the lexical
+C<no warnings;> pragma or C<no warnings 'XML::Simple';>.
 
 Two further variations are made possible by prefixing a '+' or a '-' character
 to the attribute name:
@@ -2573,9 +2629,9 @@ levels are possible:
 
 The default behaviour of C<XMLout()> is to return the XML as a string.  If you
 wish to write the XML to a file, simply supply the filename using the
-'OutputFile' option.  
+'OutputFile' option.
 
-This option also accepts an IO handle object - especially useful in Perl 5.8.0 
+This option also accepts an IO handle object - especially useful in Perl 5.8.0
 and later for output using an encoding other than UTF-8, eg:
 
   open my $fh, '>:encoding(iso-8859-1)', $path or die "open($path): $!";
@@ -2616,7 +2672,12 @@ file is assumed to be in the current directory.
 
 If the first parameter to C<XMLin()> is undefined, the default SearchPath
 will contain only the directory in which the script itself is located.
-Otherwise the default SearchPath will be empty.  
+Otherwise the default SearchPath will be empty.
+
+=head2 StrictMode => 1 | 0  I<# in+out seldom used>
+
+This option allows you to turn L<STRICT MODE> on or off for a particular call,
+regardless of whether it was enabled at the time XML::Simple was loaded.
 
 =head2 SuppressEmpty => 1 | '' | undef I<# in+out - handy>
 
@@ -2891,18 +2952,18 @@ KeyAttr hash.
 
 Data error - KeyAttr is set to say { part => 'partnum' } but the XML contains
 one or more E<lt>partE<gt> elements without a 'partnum' attribute (or nested
-element).  Note: if strict mode is not set but -w is, this condition triggers a
-warning.
+element).  Note: if strict mode is not set but C<use warnings;> is in force,
+this condition triggers a warning.
 
-=item * 
+=item *
 
 Data error - as above, but non-unique values are present in the key attribute
 (eg: more than one E<lt>partE<gt> element with the same partnum).  This will
 also trigger a warning if strict mode is not enabled.
 
-=item * 
+=item *
 
-Data error - as above, but value of key attribute (eg: partnum) is not a 
+Data error - as above, but value of key attribute (eg: partnum) is not a
 scalar string (due to nested elements etc).  This will also trigger a warning
 if strict mode is not enabled.
 
@@ -2911,7 +2972,7 @@ if strict mode is not enabled.
 =head1 SAX SUPPORT
 
 From version 1.08_01, B<XML::Simple> includes support for SAX (the Simple API
-for XML) - specifically SAX2. 
+for XML) - specifically SAX2.
 
 In a typical SAX application, an XML parser (or SAX 'driver') module generates
 SAX events (start of element, character data, end of element, etc) as it parses
@@ -3033,7 +3094,7 @@ If the 'preferred parser' is set to the string 'XML::Parser', then
 L<XML::Parser> will be used (or C<XMLin()> will die if L<XML::Parser> is not
 installed).
 
-=item * 
+=item *
 
 If the 'preferred parser' is set to some other value, then it is assumed to be
 the name of a SAX parser module and is passed to L<XML::SAX::ParserFactory.>
@@ -3180,7 +3241,7 @@ The <anon> tag can be used to form anonymous arrays:
                 ]
     }
 
-Anonymous arrays can be nested to arbirtrary levels and as a special case, if
+Anonymous arrays can be nested to arbitrary levels and as a special case, if
 the surrounding tags for an XML document contain only an anonymous array the
 arrayref will be returned directly rather than the usual hashref:
 
@@ -3227,7 +3288,7 @@ assumptions on your behalf.  These include:
 
 You're not interested in text content consisting only of whitespace
 
-=item * 
+=item *
 
 You don't mind that when things get slurped into a hash the order is lost
 
@@ -3246,19 +3307,19 @@ You don't need help converting between different encodings
 =back
 
 In a serious XML project, you'll probably outgrow these assumptions fairly
-quickly.  This section of the document used to offer some advice on chosing a
+quickly.  This section of the document used to offer some advice on choosing a
 more powerful option.  That advice has now grown into the 'Perl-XML FAQ'
 document which you can find at: L<http://perl-xml.sourceforge.net/faq/>
 
 The advice in the FAQ boils down to a quick explanation of tree versus
 event based parsers and then recommends:
 
-For event based parsing, use SAX (do not set out to write any new code for 
-XML::Parser's handler API - it is obselete).
+For event based parsing, use SAX (do not set out to write any new code for
+XML::Parser's handler API - it is obsolete).
 
 For tree-based parsing, you could choose between the 'Perlish' approach of
 L<XML::Twig> and more standards based DOM implementations - preferably one with
-XPath support.
+XPath support such as L<XML::LibXML>.
 
 
 =head1 SEE ALSO
@@ -3272,12 +3333,12 @@ The optional caching functions require L<Storable>.
 Answers to Frequently Asked Questions about XML::Simple are bundled with this
 distribution as: L<XML::Simple::FAQ>
 
-=head1 COPYRIGHT 
+=head1 COPYRIGHT
 
 Copyright 1999-2004 Grant McLean E<lt>grantm@cpan.orgE<gt>
 
 This library is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself. 
+under the same terms as Perl itself.
 
 =cut
 
